/* -*- Mode: C; c-file-style: "bsd"; -*- */
#ifndef __ELEMENT_TYPE_H__
#define __ELEMENT_TYPE_H__

#define NXP_ELEMENT_TYPE(x) x->type_info->type

/* forward declarations */
typedef struct _NxpElementTypeInfo	NxpElementTypeInfo;
typedef        int                      NxpElementType;

// NOTE: If you change the order types are initialized on types/types.c, you
// ____must____ change the values on these defines that are used only for convenience
#define NXP_ELEMENT_TYPE_VOID      1
#define NXP_ELEMENT_TYPE_INT       2
#define NXP_ELEMENT_TYPE_FRAC      3
#define NXP_ELEMENT_TYPE_REAL      4
#define NXP_ELEMENT_TYPE_COMPLEX   5
#define NXP_ELEMENT_TYPE_VECTOR    6
#define NXP_ELEMENT_TYPE_MATRIX    7
#define NXP_ELEMENT_TYPE_STRING    8
#define NXP_ELEMENT_TYPE_VARIABLE  9
#define NXP_ELEMENT_TYPE_FUNCTION 10
#define NXP_ELEMENT_TYPE_TREE     11

#define NXP_ELEMENT_TYPE_ANY  ((guint16)0xffff)
#define NXP_ELEMENT_TYPE_NONE ((guint16)0)

/* -------------------- */
#include "identifier.h"
#include "element.h"
/* -------------------- */

typedef void        (*NxpElementDestroyFunc)	(NxpElement *elem);
typedef GString*    (*NxpElementReprFunc)	(NxpElement *elem, int *precedence);
typedef GString*    (*NxpElementStrFunc)	(NxpElement *elem, int *precedence);
typedef NxpElement* (*NxpElementEvalFunc)	(NxpElement *elem, 
						 NxpEvalContext *context,
						 GError **error);
typedef NxpElement* (*NxpElementSimplifyFunc)	(NxpElement *elem, 
						 NxpEvalContext *context,
						 GError **error);
typedef NxpElement* (*NxpElementDeriveFunc)	(NxpElement *elem,
						 NxpIdentifier *var,
						 NxpEvalContext *context,
						 GError **error);
typedef gboolean    (*NxpElementTruthFunc)	(NxpElement *elem, 
						 NxpEvalContext *context, 
						 GError **error);

/* This function should recursively visit all elements contained in
 * elem. */
typedef void 	    (*NxpElementIteratorFunc)	(NxpElement *elem,
                                                 NxpElementVisitorFunc func,
						 gpointer user_data,
                                                 NxpEvalContext *context);

struct _NxpElementTypeInfo {
    int type;
    GHashTable                  *user_data;
    /*< public >*/
    char *name;
    NxpElementDestroyFunc	destroy_func;
    NxpElementEvalFunc		eval_func;
    NxpElementSimplifyFunc	simplify_func;
    NxpElementDeriveFunc	derive_func;
    NxpElementIteratorFunc	iterator_func;
    NxpElementReprFunc		repr_func;
    NxpElementStrFunc		str_func;
    NxpElementTruthFunc		truth_func;
};


void                nxp_element_type_init     (void);
void                nxp_element_type_fini     (void);
NxpElementTypeInfo* nxp_element_type_get_info (int type);

int                 nxp_element_next_type     (void);

gpointer            nxp_element_type_get_data (NxpElementTypeInfo  *type_info,
					       const char* property_name);

void                nxp_element_type_set_data (NxpElementTypeInfo  *type_info,
					       const char* property_name,
					       gpointer data);

#endif /* __ELEMENT_TYPE_H__ */




