/* -*- Mode: C; c-file-style: "stroustrup" -*- */
#ifndef __MODULE_H__
#define __MODULE_H__

#include <gmodule.h>

#include "config.h"
#ifdef HAVE_DB4_DB_H
# include <db4/db.h>
#else
# include <db.h>
#endif

typedef struct _NxpModule 	 NxpModule;
typedef struct _NxpModuleEntry 	 NxpModuleEntry;
typedef struct _NxpModulePrivate NxpModulePrivate;

/* modules must define a function named "nxp_module_init" with the
   following prototype: */
#include "eval_context.h"
typedef void (*NxpModuleInitFunc) (NxpEvalContext *context, GError **error);


struct _NxpModule {
    char             *name;
    char             *filename;
    GModule          *module;
    NxpModulePrivate *priv;
};

extern NxpModule *nxp_current_module;
extern GPtrArray *nxp_loaded_modules; /* array<NxpModule *> */


struct _NxpModuleEntry {
    char     *name;		/* module base name */
    char     *location;		/* full path name of the so file */
    char     *description;	/* can be NULL */
    gboolean  loaded:1;		/* TRUE if module is currently loaded */
    char     *longer_desc;	/* A longer description (like an introduction to the module) */
};


void             nxp_module_init                (NxpEvalContext  *context,
						 GError         **error);
NxpModule*       nxp_module_load                (const char      *name,
						 NxpEvalContext  *context);
void             nxp_module_rebuild_search_path (void);
char*            nxp_module_help                (NxpModule       *module,
						 const char      *full_name);
char*            nxp_help                       (const char      *topic,
						 NxpEvalContext  *context);
NxpModuleEntry** nxp_modules_listing            (void);
void             nxp_modules_list_free          (NxpModuleEntry **entries);
int              open_db_file                   (const gchar     *dbfilename,
						 DB             **db);
void             nxp_open_help_db               (DB             **db);
GList*           nxp_help_list                  (NxpEvalContext  *context);

#endif
